#!/bin/bash
#
# Copyright (C) 2023, Max Planck Institute for Intelligent Systems, Tuebingen, Germany
#
# Download BEDLAM image data. Resume download if interrupted.
#
# Total data size: ~6TB
# Framerate: 30fps
# Image size: 1280x720
#
# Requirements: wget (`apt install wget`)
#
# Usage: 
#   1. Register at https://bedlam.is.tuebingen.mpg.de/ with
#      `email` and `password`
#   2. Read and accept license conditions
#      + https://bedlam.is.tuebingen.mpg.de/license.html
#   3. Make `bedlam_download` folder on target drive. Make sure
#      that you have enough disk space on that drive.
#   4. Copy this script and `b0.txt` file to `bedlam_download` folder
#   5. Run this script with desired target data type from that folder 
#      to download data.
#      Do not use `all` if you don't need depth data.
#      We recommend to start with smallest folder first. 
#      + `all`:   will download everything (depth,gt,masks,mp4,png), ~6TB local space needed
#      + `depth`: depth images (EXR), ~3.8TB
#      + `gt`:    scene ground truth (CSV), ~100MB
#      + `masks`: segmentation masks (PNG), ~30GB
#      + `mp4`:   movies (MP4), ~20GB
#      + `png`:   image sequences (PNG), ~2.2TB
#
#      Example: `bash ./be_download.sh mp4`
#
#   6. Input the `email` and `password` you used for registration when prompted
#   7. Wait for download to finish. You can rerun script to continue interrupted transfers.
#   8. Run be_validate_download.sh script to validate downloads. See header for instructions.
#
# Notes:
#   + Windows
#     + Use WSL2 (Ubuntu 22.04)
#

# Globals
DATADIRS_PATH="./b0.txt"

# Return URL encoded version of input string
urle () { [[ "${1}" ]] || return 1; local LANG=C i x; for (( i = 0; i < ${#1}; i++ )); do x="${1:i:1}"; [[ "${x}" == [a-zA-Z0-9.~-] ]] && echo -n "${x}" || printf '%%%02X' "'${x}";done; echo; }

function download_file {
    local username=$1
    local password=$2 
    local url=$3
    local filename=$4
    wget --post-data "username=$username&password=$password" $url -O $filename --no-check-certificate --continue
    return $?
}

######################################################################
# Main
######################################################################

echo "BEDLAM data download tool" 1>&2
if [ -z "$1" ]; then
    echo 1>&2
    echo "Usage: $0 all|depth|gt|masks|mp4|png" 1>&2
    echo 1>&2
    echo "Example: $0 png" 1>&2

    exit 1
fi

datatype=$1

if [[ "$datatype" != "all" && "$datatype" != "depth" && "$datatype" != "gt" && "$datatype" != "masks" && "$datatype" != "mp4" && "$datatype" != "png" ]]; then
    echo "ERROR: Invalid datatype: $datatype" 1>&2
    echo "Usage: $0 all|depth|gt|masks|mp4|png" 1>&2
    exit 1
fi

# Check if data directory file is found
echo "BEDLAM data directories file: '$DATADIRS_PATH'" 1>&2

if [ ! -f "$DATADIRS_PATH" ]; then
    echo "ERROR: Data directories file not found" 1>&2
    exit 1
fi

if [ "$datatype" == "all" ]; then
    target_types=("gt" "png" "mp4" "masks" "depth") # smallest size first
else
    target_types=("$datatype")
fi

read -p "Enter your BEDLAM website email address: " username
read -p "Enter your BEDLAM website password: " password

username=$(urle $username)
password=$(urle $password)

# Download specified data
for type in "${target_types[@]}"
do
    echo "Downloading data type: $type"

    # Process specified directories
    while read datadir
    do
        echo "  Processing: $datadir"

        typefolder="$type"

        if [[ "$type" == "depth" || "$type" == "png" ]]; then
            archive_ids=(0 1 2 3 4)
            for archive_id in "${archive_ids[@]}"
            do
                filename="${datadir}_${type}.${archive_id}.tar"
                url="https://download.is.tue.mpg.de/download.php?domain=bedlam&resume=1&sfile=b0/${datadir}/${typefolder}/${filename}"
                download_file $username $password $url $filename
            done
        else
            filename="${datadir}_${type}.tar"

            if [ "$type" == "gt" ]; then
                typefolder="ground_truth"
                filename="${filename}.gz"
            fi

            url="https://download.is.tue.mpg.de/download.php?domain=bedlam&resume=1&sfile=b0/${datadir}/${typefolder}/${filename}"
            download_file $username $password $url $filename
        fi

    done < $DATADIRS_PATH

done
