#!/bin/bash
#
# Copyright (C) 2023, Max Planck Institute for Intelligent Systems, Tuebingen, Germany
#
# Validate downloaded BEDLAM data
#
# Usage: Copy this script and relevant b0_checksums_*.xxh128 files to BEDLAM
#        download directory and run script from there.
#        Example: `bash ./be_validate_download.sh mp4`
#
# Requirements:
# + xxhsum (version 0.8.1 or higher)
#   + https://github.com/Cyan4973/xxHash
#   + Linux
#     + Available in Ubuntu 22.04+ via `apt install xxhash`
#   + Windows
#     + We recommend to run script via Git for Windows BASH for best 
#       performance if data is on fast NTFS drive
#       1. Install https://gitforwindows.org/ to default path
#       2. Download precompiled `xxhsum.exe` binary from xxHash GitHub repo and
#          copy it to C:\Program Files\Git\usr\bin
#     + Slower alternative: WSL2 (Ubuntu 22.04)
#

# Globals
CHECKSUM_PATH_PREFIX="./b0_checksums"

######################################################################
# Main
######################################################################

if [ -z "$1" ]; then
    echo "Validate BEDLAM download data" 1>&2
    echo 1>&2
    echo "Usage: $0 all|depth|gt|masks|mp4|png" 1>&2
    echo 1>&2
    echo "Example: $0 png" 1>&2

    exit 1
fi

CHECKSUM_PATH="${CHECKSUM_PATH_PREFIX}_$1.xxh128"

# Check if xxhsum checksum tool is available and at proper version 0.8.1+
xxhsum_version=$(xxhsum --version 2>&1 | grep -oP 'xxhsum(?:\.exe)? \K[0-9.]+') # xxhsum outputs version to stderr, reports version as xxhsum.exe on windows
if [ -z $xxhsum_version ]; then
    echo "ERROR: Checksum tool not installed: 'xxhsum'. Please check instructions in this script header." 1>&2
    exit 1
fi

xxhsum_version_major=$(echo $xxhsum_version | awk -F. '{print $1}')
xxhsum_version_minor=$(echo $xxhsum_version | awk -F. '{print $2}')
xxhsum_version_patch=$(echo $xxhsum_version | awk -F. '{print $3}')

if [ "$xxhsum_version_minor" -lt 8 ] || [ "$xxhsum_version_minor" -eq 8 -a "$xxhsum_version_patch" -lt 1 ]; then
    echo "ERROR: Outdated checksum tool version: '$xxhsum_version'. Please check instructions in this script header." 1>&2
    exit 1
fi

# Check if checksum file is found
echo "Checksum path: '$CHECKSUM_PATH'" 1>&2

if [ ! -f "$CHECKSUM_PATH" ]; then
    echo "ERROR: Checksum file not found" 1>&2
    exit 1
fi

# Process checksums
xxhsum -c $CHECKSUM_PATH
exit_status=$?
echo Checksum validation exit status: $exit_status
if [ $exit_status -eq 0 ]; then
    echo "Validation successful" 1>&2
    exit 0
else
    echo "ERROR: Validation failed for one or more files" 1>&2
    exit 1
fi

